from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models


from ..mixins import (
    AvatarMixin,
    BasicUserFieldsMixin,
    DateJoinedUserMixin,
    EmailUserMixin,
    EmailVerifyUserMixin,
    IsStaffUserMixin,
    NameUserMethodsMixin,
    VerifyEmailMixin,
)


class User(AvatarMixin, VerifyEmailMixin, PermissionsMixin, AbstractBaseUser):
    pass


class BasicUser(BasicUserFieldsMixin, AbstractBaseUser):
    pass


class VerifyEmailUser(VerifyEmailMixin):
    pass


class CustomBasicUserFieldsMixin(
        NameUserMethodsMixin, EmailUserMixin, DateJoinedUserMixin,
        IsStaffUserMixin):
    name = models.TextField()

    USERNAME_FIELD = 'email'

    class Meta:
        abstract = True


class CustomNameUser(
        AvatarMixin, EmailVerifyUserMixin, CustomBasicUserFieldsMixin,
        AbstractBaseUser):
    pass



from incuna_test_utils.compat import wipe_id_fields_on_django_lt_17

from user_management.models.tests import factories, utils
from ..models import AuthToken


class TestAuthToken(utils.APIRequestTestCase):
    model = AuthToken

    def test_fields(self):
        fields = self.model._meta.get_all_field_names()

        expected = wipe_id_fields_on_django_lt_17((
            # Inherited from subclassed model
            'key',
            'user',
            'user_id',
            'created',

            'expires',
        ))

        self.assertCountEqual(fields, expected)

    def test_unicode(self):
        uni_key = 'OSSUM'
        obj = self.model(key=uni_key)
        self.assertEqual(str(obj), uni_key)

    def test_multiple_tokens(self):
        user = factories.UserFactory.create()
        tokens = factories.AuthTokenFactory.create_batch(2, user=user)

        expected_tokens = self.model.objects.filter(user=user)

        self.assertCountEqual(tokens, expected_tokens)



from django.core.management.base import BaseCommand
from django.utils import timezone

from user_management.api.models import AuthToken


class Command(BaseCommand):
    help = "Remove expired auth tokens from the database."

    def handle(self, *args, **options):
        now = timezone.now()
        AuthToken.objects.filter(expires__lte=now).delete()
